import numpy as np
from math import ceil

def identify_switching_current(ib_max,ib_steps,vs):
	ibs_diff = np.linspace( -ib_max + 2*ib_max/ib_steps , ib_max - 2*ib_max/ib_steps , ib_steps - 1)

	v_diff = np.diff(vs)
	n = ceil((ib_steps-1)/2)

	v_diff_neg = v_diff[:n]
	v_diff_pos = v_diff[n:]
	
	return [ibs_diff[np.argmax(v_diff_neg)], ibs_diff[n+np.argmax(v_diff_pos)]]

def identify_switching_current_2(ibs,vs,n_0,n_1):

    vs_range = vs[n_0:n_1]
    ibs_range = ibs[n_0:n_1]
    ibs_diff = np.linspace( ibs[n_0] + (ibs[n_0+1]-ibs[n_0])/2 , ibs[n_1-1] - (ibs[n_0+1]-ibs[n_0])/2 , n_1 - n_0 - 1)

    v_diff = np.diff(vs_range) 
    
    return ibs_diff[np.argmax(v_diff)] 

def box(x,a,b):
	if a < x < b:
		return 1
	else:
		return 0

#averages over output of func of single variable with integer input (0 to ens_size - 1) 
def ensemble_avg(ens_size, func):
    
    data = np.array(func(0))
    for i in range(1,ens_size):
        data = data + np.array(func(i))

    return data/ens_size 

#calculate mean voltage for ensemble of trajcetories with rtj_of_p = rtj_of_p(p,dummy)
def means_of_param_p(ens_size,ps,rtj_of_ib):

    means = np.array([np.zeros(len(ps)),np.zeros(len(ps))])

    for i in range(0,len(ps)):
        #print(i)
        rtj = lambda x: rtj_of_ib(p[i],x)
        temp_data = ensemble_avg(ens_size, rtj)
        N = len(temp_data[0])
        means[i,0] = np.sum(temp_data[0,ceil(0.2*N):N+1])/(N+1 - ceil(0.2*N))
        means[i,1] = np.sum(temp_data[0,ceil(0.2*N):N+1])/(N+1 - ceil(0.2*N))

    return means