import sys
sys.path.insert(0, '..')

import numpy as np
import time
from scipy.optimize import fsolve

from experimental_data.process_exp_data import process_exp_data_func
import library.sde_euler_maruyama as sde
import sweep_forward_backward as sweeps

#note on units:
#
#tau = omega_p t with omega_p^2 = 2eI_c / hbar C
#v = \partial_tau \phi
#Q_tun = R_tun C omega_p
#
#all times in units of omega_p^(-1), currents in units of I_c, all energies in units of hbar*omega_p
#other expressions, see notes

#main
def main(run_index , params):
    
    small = 10**(-10)

    I_qps = process_exp_data_func()
        
    #def params of simulation
    if len(params) > 0:
        theta,ramp_rate,I_c,hbar_omega_p,tau_KM,Q_KM,theta_KM,dt,ib_max,ib_steps = [float(param) for param in params]
        which_data = 1
        ib_steps = int(ib_steps)
        
    else:
        #physical parameters
        theta = 0.5
        ramp_rate = 10**(-3)
        I_c = 100 #in nA
        hbar_omega_p = 0.3 #in units of meV, necessary to convert V to v: V/mV = (hbar_omega_p/meV) v/2 == hbar_omega_p v/2
        
        #Kauz Martinis parameters
        tau_KM = 1000
        Q_KM = 10
        theta_KM = 0.5

        #time discretization
        dt = 0.01 #size of time steps, should be << 1
        
        #ib sweep params
        ib_max = 0.25 #range of ibs
        ib_steps = 500 #number of data points
    	
	#which element
        which_data = 1

    #time per ib step and corresponding number of discrete times
    T = 2*ib_max/((ib_steps-1)*ramp_rate)
    N = int(T/dt) 

    print('')
    print('time spent at a single ib and for full sweep (one direction)')
    print([T,T*ib_steps])

    #normal state resistance and corresponding Q_N
    R_N = 20000
    Q_N = 2*I_c*R_N*10**(-6)/hbar_omega_p

    
                
    print('Q at small v vs large v vs 20kOhm')
    print([I_c*0.01/I_qps[which_data](hbar_omega_p*0.01/2) , I_c*(2*5/hbar_omega_p)/I_qps[which_data](5) , Q_N ])
    
    #qp current and noise 
    i_qp = lambda v: I_qps[which_data](hbar_omega_p*v/2)/I_c
    #noise_qp = lambda v: 2*theta*np.abs( ((i_qp(v) + i_qp(-v))/2) + small)/np.abs(v + 0.3*small)
    #noise_qp = lambda v: 2*theta*np.abs(i_qp(v) + small)/np.abs(v + 0.3*small)
    noise_qp = lambda v: 2*theta/150

    #variables X[0] = phi, X[1] = v, X[2] = v_KM
    F_of_ib = lambda ib,t,X: np.array([ X[1] , ib - np.sin(X[0]) - ( i_qp(X[1]) + (X[1]-X[2])/Q_KM )  , (X[1]-X[2])/tau_KM ])
    G = lambda t,X: np.array([  [0,0] , 
                                [ np.sqrt(noise_qp(X[1])), np.sqrt(2*theta_KM/Q_KM) ] ,
                                [0,np.sqrt(2*theta_KM*Q_KM)/tau_KM ] 
                            ])
    
        
    [v0] = fsolve(lambda v: -ib_max - i_qp(v) , Q_N*ib_max) + np.random.normal(loc=0.0, scale= small)
    vmeans_forward , vmeans_backward = sweeps.sweep_forward_backward_KM(dt,T,v0,ib_max,ib_steps,F_of_ib,G)

    filename = "data/RCSJ_sweep_experimental_data_mn_symmetric_noise_KM_theta={}_ramp_rate={}_I_c={}_hbar_omega_p={}_tau_KM={}_Q_KM={}_theta_KM={}_Q_noise=100_(dt={}_ib_max={}_ib_steps={})_run={}.npy".format(
                            theta,ramp_rate,I_c,hbar_omega_p,tau_KM,Q_KM,theta_KM,dt,ib_max,ib_steps,run_index)
    np.save( filename , [ [theta,ramp_rate,I_c,hbar_omega_p,tau_KM,Q_KM,theta_KM,dt,ib_max,ib_steps] , vmeans_forward , vmeans_backward ] )

    return 0

if __name__== "__main__":
    #if the file is called without arguments, sets run_index to default value, else uses index from argument
    if len( sys.argv ) == 1:
        run_index = 0
        params = []
    else:
        run_index = sys.argv[1]
        if len( sys.argv ) > 2:
            params = sys.argv[2:] 
        else:
            params = []

    main( run_index , params )
