import sys
sys.path.insert(0, '..') 
sys.path.insert(0, '../..') 
import pathlib

import numpy as np
from scipy.optimize import curve_fit
from scipy.interpolate import UnivariateSpline
from scipy.ndimage import gaussian_filter1d

import matplotlib.pyplot as plt
#from matplotlib import rc
#rc('font', **{'family': 'serif', 'serif': ['Computer Modern']})
#rc('text', usetex=True)
import time

#note on units:
#
#tau = omega_p t with omega_p^2 = 2eI_c / hbar C
#v = \partial_tau \phi
#Q_tun = R_tun C omega_p
#
#all times in units of omega_p^(-1), currents in units of I_c, all energies in units of hbar*omega_p
#other expressions, see notes

def process_exp_data_func():
    #import experimental data    
    filenames = [str(pathlib.Path(__file__).parent.resolve()) + '/didV_00049_Pb20kOhm.txt',
                 str(pathlib.Path(__file__).parent.resolve()) + '/didV_00047_Mn20kOhm.txt',
                 str(pathlib.Path(__file__).parent.resolve()) + '/didV_00052_Cr20kOhm.txt']
                 
    data = []
    for filename in filenames:
        
        f = open(filename, 'r')
        temp_data = f.readlines()
        f.close()

        temp_data = [line.split() for line in temp_data]
        while not(temp_data[0][0][0].isdigit()):
            del temp_data[0]
        temp_data.reverse()

        temp_data = np.array([ [ float(el) for el in line ] for line in temp_data ]).T

        data.append(temp_data)    

    #I(V) data contains Josephson contribution at small V: I(V) = I_JJ(V) + I_qp(V)
    #fit J peak in dIdV, subtract, integrate to obtain I_qp(V)
    #do this for all three cases (Pb,Mn,Cr)
    
    I_qp_splines = []
    
    #voltage range
    V_lim = 5
    n_V_steps = 1000
    Vs = np.linspace(-V_lim,V_lim,n_V_steps)

    '''
    plt.figure(1)
    plt.plot(data[0][0] + 0.01865 , data[0][1] , label='backward' , color = 'red')
    plt.plot(data[0][0] - 0.0034 , -np.flip(data[0][1]) , label='backward' , color = 'blue')
    plt.axhline(0 , color = 'black' , linewidth = 0.5 )
    plt.axvline(0, color = 'black' , linewidth = 0.5  )
    plt.show(block=True)
    '''

    for which_data in range(0,3):
        #which_data = 2
        print(' ')
        print(['Pb:','Mn:','Cr:'][which_data])

        V_data , I_data , dIdV_data = data[which_data]
        #asym = [0,1,1][which_data]

        #define range of voltage for fitting of Josephson peak
        fit_range = [27,27,27][which_data]
        fit_limit_bot = 499-fit_range
        fit_limit_top = 499+fit_range

        V_data_cut , I_data_cut ,  dIdV_data_cut = [ V_data[fit_limit_bot:fit_limit_top]  , I_data[fit_limit_bot:fit_limit_top] , dIdV_data[fit_limit_bot:fit_limit_top] ]

        #P(E) fit form of dIdV Josephson peak including V^3 term (second line with constant background)
        I_J = lambda V,A,B,C: ( 0*V
            + A*B*V/(B**2 + V**2)  
            + C*B*V**3/(B**2 + V**2)**2    
            )

        I_QP = lambda V,S1,AS2,S3,AS4: ( np.abs(S1)*V 
            + AS2*V**2 
            + S3*V**3 
            #+ AS4*V**4
            )

        I_J_QP = lambda V,A,B,C,S1,AS2,S3,AS4: I_J(V,A,B,C) + I_QP(V,S1,AS2,S3,AS4)  
        I_J_QP_V_shift_bg = lambda V,A,B,C,S1,AS2,S3,AS4,V_shift,I_BG: I_J_QP(V + V_shift,A,B,C,S1,AS2,S3,AS4) + I_BG

        #fit the data cut [A,B,C,S1,AS2,S3,AS4,V_shift,I_BG]
        p_init = [
            [20,0.13,-20,5,0,0,0,0,0],
            [8,0.14,-11,6,-2,0,0,0,0],
            [6,0.14,-8,5,4.5,1.5,0,0,0]
            ][which_data]


        [A,B,C,S1,AS2,S3,AS4,V_shift,I_BG], pcov2 = curve_fit( I_J_QP_V_shift_bg, V_data_cut, I_data_cut, p0 = p_init )

        print('')
        print('voltage and current offset:')
        print([V_shift,I_BG])
        
        print('')
        print('fit params [A , B , C , S1, AS2 ]:')
        print([A , B , C , S1, AS2])
        print('[S3, AS4 ]:')
        print([S3,AS4])

        #account for voltage and current offset
        V_data = V_data + V_shift
        V_data_cut = V_data_cut + V_shift
        I_data = I_data - I_BG
        I_data_cut = I_data_cut - I_BG

        print('')
        print('fit voltage range')
        print([V_data[fit_limit_bot],V_data[fit_limit_top-1]])

        #subtract fit from data and smoothen, then spline
        I_data_smooth = gaussian_filter1d(I_data  -  I_J(V_data,A,B,C), sigma=5, mode='mirror') 
        #I_data_smooth[fit_limit_bot:fit_limit_top] = I_bg(V_data_cut,S1, S3, AS2, AS4)
        I_qp_splines.append( UnivariateSpline(V_data, I_data_smooth, k = 1, s = 0) ) #interpolate with lines
        
        #mid-gap and normal state resistance
        #R_0 = (10**6)/S1
        #R_N = (10**6)*V_lim/(I_qp_splines[-1](V_lim)-I_qp_splines[-1](0))

        #print('')
        #print('resistance at zero and large V')
        #print([R_0,R_N])

        '''
        ext_lim = 50
        V_data_cut_extended = V_data[500 - ext_lim:500 + ext_lim]
        I_data_cut_extended = I_data[500 - ext_lim:500 + ext_lim]
        I_data_smooth_cut_extended = I_data_smooth[500 - ext_lim:500 + ext_lim]

        plt.figure(2*which_data + 1)

        plt.plot( V_data_cut_extended - V_shift , I_data_cut_extended + I_BG, label=r'raw data' , color = 'red', alpha = 0.3 )
        plt.plot( V_data_cut_extended , I_data_cut_extended , label=r'corrected offset data' , color = 'red' )
        plt.plot( V_data_cut_extended , I_data_cut_extended - I_J(V_data_cut_extended,A,B,C) , label=r'subtracted Josephson data' , color = 'orange')
        
        plt.plot( V_data_cut - V_shift , I_J_QP_V_shift_bg(V_data_cut - V_shift,A,B,C,S1,AS2,S3,AS4,V_shift,I_BG)  , 
            label=r'$I_\mathrm{meas}(V) $' , color = 'blue', alpha = 0.3)
        plt.plot( V_data_cut , I_J_QP_V_shift_bg(V_data_cut - V_shift,A,B,C,S1,AS2,S3,AS4,V_shift,I_BG) - I_BG , 
            label=r'$I_\mathrm{meas}(V - V_\mathrm{bg}) - I_\mathrm{bg}$' , color = 'blue')

        plt.plot( V_data_cut_extended , I_qp_splines[-1](V_data_cut_extended) - I_qp_splines[-1](0) , label=r'$I_\mathrm{qp}(V)$' , color = 'black')
        
        fontsize1 = 13
        fontsize2 = 20
        plt.axhline(0 , color = 'black' , linewidth = 0.5 )
        plt.axvline(0, color = 'black' , linewidth = 0.5  )
        plt.ylabel(r'$I$ [nA]',fontsize=fontsize2)
        plt.xlabel(r'$V$ [mV]',fontsize=fontsize2)
        plt.xlim([-0.5, 0.5])
        plt.ylim([[-6,-5,-2.5][which_data],[6,3.5,3.5][which_data]])
        if which_data == 0:
            plt.legend(loc=["upper left","lower right","upper left"][which_data], prop={'size': fontsize1} ,frameon=False)
        plt.text( [0.25,0.25,0.25][which_data] , [4.8,2.65,2.9][which_data] , [r'(a) Pb' ,r'(e) Mn' , r'(c) Cr'][which_data] , fontsize=fontsize2 )
        plt.tick_params(
        axis='x',           
        which='both', 
        labelsize=fontsize1)
        plt.tick_params(
        axis='y',         
        which='both',
        labelsize=fontsize1)
        plt.tight_layout()
        filename = ["../figures/fit_pb_small.png","../figures/fit_mn_small.png","../figures/fit_cr_small.png"][which_data]
        plt.savefig(filename)
        #plt.show(block=True)
                
        
        plt.figure(2*which_data + 2)
        Vs = np.linspace(0,6,2000)
        plt.plot(V_data[499:] , I_data[499:]  , label=r'corrected offset data (+)'  , color = 'red' )
        plt.plot(-(V_data[:499])[::-1] , -(I_data[:499])[::-1] , linestyle = 'dashed' , label=r'corrected offset data (-)' , color = 'red' )
        plt.plot(Vs  ,  I_qp_splines[-1](Vs) - I_qp_splines[-1](0)   , label=r'$I_\mathrm{qp}(V)$' , color = 'black')
        plt.plot(Vs  ,  -(I_qp_splines[-1](-Vs) - I_qp_splines[-1](0) ) , linestyle = 'dashed'   , label=r'$-I_\mathrm{qp}(-V)$' , color = 'black')
        plt.plot(Vs  ,  np.abs( I_qp_splines[-1](Vs) - I_qp_splines[-1](0)  + (I_qp_splines[-1](-Vs) - I_qp_splines[-1](0) ) )  , label=r'$\vert I_\mathrm{qp}(V) + I_\mathrm{qp}(-V) \vert$' , color = 'grey')

        plt.axhline(0 , color = 'black' , linewidth = 0.5 )
        plt.axvline(0, color = 'black' , linewidth = 0.5  )
        plt.xlim([0, 2])
        plt.ylim([0, [60,70,60][which_data]])
        plt.ylabel(r'$I$ [nA]',fontsize=fontsize2)
        plt.xlabel(r'$V$ [mV]',fontsize=fontsize2)
        if which_data == 0:
            plt.legend(loc="upper left", prop={'size': fontsize1} ,frameon=False )
        plt.text( [1.5,1.5,1.5][which_data] , [54,63,54][which_data] , [r'(b) Pb' ,r'(f) Mn' , r'(d) Cr'][which_data] , fontsize=fontsize2 )
        plt.tick_params(
        axis='x',           
        which='both', 
        labelsize=fontsize1)
        plt.tick_params(
        axis='y',         
        which='both',
        labelsize=fontsize1)
        plt.tight_layout()
        filename = ["../figures/fit_pb_large.png","../figures/fit_mn_large.png","../figures/fit_cr_large.png"][which_data]
        plt.savefig(filename)
        #plt.show(block=True)

    plt.show(block=True)
    #'''

    #make sure that I(0) = 0
    I_qps = [lambda V: I_qp_splines[0](V) - I_qp_splines[0](0) , lambda V: I_qp_splines[1](V) - I_qp_splines[1](0) , lambda V: I_qp_splines[2](V) - I_qp_splines[2](0) ]
    #I_qps = [lambda V: (I_qp_splines[0](V) - I_qp_splines[0](-V))/2 , lambda V: I_qp_splines[1](V) - I_qp_splines[1](0) , lambda V: I_qp_splines[2](V) - I_qp_splines[2](0) ]
    
    return I_qps

#main
def main(run_index , params):

    I_qps = process_exp_data_func()
    
    #'''
    cnst = I_qps[0](0)
    i_qp = lambda v: I_qps[0](v)
    i_qp_2 = lambda v: I_qps[0](v) - I_qps[0](0)
    i_qp_3 = lambda v: I_qps[0](v) + 0.008524120280399913

    print('')
    print(cnst)
    start_time = time.time() 
    a = i_qp(np.linspace(-5,5,10**7))
    print("--- %s seconds ---" % (time.time() - start_time))
    start_time = time.time() 
    a = i_qp_2(np.linspace(-5,5,10**7))
    print("--- %s seconds ---" % (time.time() - start_time))
    start_time = time.time() 
    a = i_qp_3(np.linspace(-5,5,10**7))
    print("--- %s seconds ---" % (time.time() - start_time))

    return 0
    #'''    

    V_lim = 5  
    n_V_steps = 10000 
    Vs = np.linspace(0,V_lim,n_V_steps)

    plt.figure(1)
    plt.plot(Vs , I_qps[0](Vs)  , label='backward' , color = 'red')
    plt.plot(Vs , -I_qps[0](-Vs)  , label='backward' , color = 'red')
    plt.plot(Vs , I_qps[1](Vs)  , label='backward' , color = 'blue')
    plt.plot(Vs , -I_qps[1](-Vs)  , label='backward' , color = 'blue')
    plt.plot(Vs , I_qps[2](Vs)  , label='backward' , color = 'purple')
    plt.plot(Vs , -I_qps[2](-Vs)  , label='backward' , color = 'purple')
    plt.axhline(0 , color = 'black' , linewidth = 0.5 )
    plt.axvline(0 , color = 'black' , linewidth = 0.5  )
    plt.show(block=True)
    
    plt.figure(2)
    plt.plot(Vs , (I_qps[0](Vs) + I_qps[0](-Vs) )/(I_qps[0](Vs) - I_qps[0](-Vs) ) , label='backward' , color = 'red')
    plt.plot(Vs , (I_qps[1](Vs) + I_qps[1](-Vs) )/(I_qps[1](Vs) - I_qps[1](-Vs) )   , label='backward' , color = 'blue')
    plt.plot(Vs , (I_qps[2](Vs) + I_qps[2](-Vs) )/(I_qps[2](Vs) - I_qps[2](-Vs) )  , label='backward' , color = 'purple')
    plt.axhline(0 , color = 'black' , linewidth = 0.5 )
    plt.axvline(0, color = 'black' , linewidth = 0.5  )
    plt.show(block=True)

    plt.figure(3)
    plt.plot(Vs , I_qps[0](Vs) + I_qps[0](-Vs)  , label='backward' , color = 'red')
    plt.plot(Vs , I_qps[1](Vs) + I_qps[1](-Vs)   , label='backward' , color = 'blue')
    plt.plot(Vs , I_qps[2](Vs) + I_qps[2](-Vs) , label='backward' , color = 'purple')
    plt.axhline(0 , color = 'black' , linewidth = 0.5 )
    plt.axvline(0, color = 'black' , linewidth = 0.5  )
    plt.show(block=True)

    return 0



if __name__== "__main__":
    #if the file is called without arguments, sets run_index to default value, else uses index from argument
    if len( sys.argv ) == 1:
        run_index = 0
        params = []
    else:
        run_index = sys.argv[1]
        if len( sys.argv ) > 2:
            params = sys.argv[2:] 
        else:
            params = []

    main( run_index , params )