import os.path

import numpy as np
from math import ceil

import matplotlib.pyplot as plt


def identify_switching_current(ib_max,ib_steps,vs):
	ibs_diff = np.linspace( -ib_max + 2*ib_max/ib_steps , ib_max - 2*ib_max/ib_steps , ib_steps - 1)

	v_diff = np.diff(vs)
	n = ceil((ib_steps-1)/2)

	v_diff_neg = v_diff[:n]
	v_diff_pos = v_diff[n:]
	
	return [ibs_diff[np.argmax(v_diff_neg)], ibs_diff[n+np.argmax(v_diff_pos)]]


#main
def main():
	hbar_omega_p = 0.3
	I_c = 100
	ib_max = 0.25
	ib_steps = 500

	N_runs = 100

	

	data=[]
	for run_index in range(1,N_runs + 1):
		filename = "data/RCSJ_experimental_data_KM_sweep_" + ["pb","mn","cr"][0] + "_" + str(run_index) + ".npy"
		#filename = "data/RCSJ_experimental_data_KM_sweep_pb_asym_Ic_" + str(run_index) + ".npy"
		
		if os.path.isfile(filename):
			data.append(
				np.load( filename , allow_pickle=True )
			)
		else:
			print(filename)
			print(str(run_index) + " does not exist")
	

	vmeans_forwards = np.array([ dat[1] for dat in data ])
	vmeans_backwards = np.array([ dat[2] for dat in data ])
	
	#extract switching and retrapping current
	ib_steps = int(ib_steps)
	forward_switchings = np.array([ identify_switching_current(ib_max,ib_steps,vmeans_forward) for vmeans_forward in vmeans_forwards ])
	backward_switchings = np.array([ identify_switching_current(ib_max,ib_steps,vmeans_backward) for vmeans_backward in vmeans_backwards ])

	
	#Make plots
	fontsize = 20

	fig1 , (Plt) = plt.subplots( 1 , 1 , sharex= False )
	fig1.set_size_inches([8,6])
	
	Ibs = np.linspace( -ib_max , ib_max , ib_steps)*I_c 
	plt.figure(1)
	plt.plot(Ibs , hbar_omega_p*vmeans_forwards[27].T/2  , color = 'red') 
	plt.plot(Ibs , hbar_omega_p*vmeans_backwards[27].T/2 , color = 'green') 
	plt.axhline(0 , color = 'black' , linewidth = 0.5 )
	plt.axvline(0, color = 'black' , linewidth = 0.5  )

	Plt.tick_params(
	axis='x',          
	which='both',      
	bottom=True,      
	top=False,        
	labelbottom=True,
	labelsize=fontsize)

	Plt.tick_params(
	axis='y',          
	which='both',
	labelsize=fontsize)
	
	Plt.set_ylabel('V [mV]',fontsize=fontsize)
	Plt.set_xlabel('I_b [nA]$',fontsize=fontsize)
	
	fig1.tight_layout()

		
	fig2 , (Plt) = plt.subplots( 1 , 1 , sharex= False )
	fig2.set_size_inches([8,6])

	ibs_hist = np.linspace( 0. , 0.2 , 70)

	Plt.hist(-forward_switchings[:,0], label = 'I_re^-',  bins = ibs_hist , alpha = 0.7 ) 
	Plt.hist(forward_switchings[:,1], label = 'I_sw^+', bins = ibs_hist , alpha = 0.7 ) 
	Plt.hist(-backward_switchings[:,0],  label = 'I_sw^-', bins = ibs_hist , alpha = 0.7 ) 
	Plt.hist(backward_switchings[:,1], label = 'I_re^+', bins = ibs_hist , alpha = 0.7 )  

	Plt.tick_params(
	axis='x',         
	which='both',      
	bottom=True,      
	top=False,        
	labelbottom=True,
	labelsize=fontsize)

	Plt.tick_params(
	axis='y',          
	which='both',
	labelsize=fontsize)
	Plt.set_xlim([0., 0.2])

	Plt.set_ylabel('count',fontsize=fontsize)
	Plt.set_xlabel('I_b/I_c',fontsize=fontsize)
	Plt.legend(loc="upper right", prop={'size': fontsize} )

	fig2.tight_layout()

	plt.show(block=True)

if __name__== "__main__":
  main()