import sys
sys.path.insert(0, '..') 

import numpy as np
import time

import library.sde_euler_maruyama as sde

def sweep_forward_backward(dt,T,v0,ib_max,ib_steps,gen_force_ib,noise_amp):
    
    start_time = time.time() 

    #ib array and data container
    N = int(T/dt)
    ib_steps = int(ib_steps)
    ibs = np.linspace( -ib_max , ib_max , ib_steps) 
    vmeans_forward = np.zeros(ib_steps)
    vmeans_backward = np.zeros(ib_steps)

    #initial values (start with ohm velocity!)
    phi0 = 0
    v0 = v0

    print('')

    #forward sweep
    for j in range(ib_steps):

        gen_force = lambda t,phi,v: gen_force_ib(ibs[j],t,phi,v)
        
        data = sde.run_trajectory(dt,T,gen_force,noise_amp, x0 = phi0, v0 = v0)

        #new initial values (s.t. hysteresis may be observed)
        phi0 = data[0,-1]
        v0 = data[1,-1]

        vmeans_forward[j] = np.sum(data[1])/N

        if j%50 == 0:
            print("forward "+str(j))

    #backward sweep
    for j in range(ib_steps):

        gen_force = lambda t,phi,v: gen_force_ib(ibs[ib_steps-j-1],t,phi,v)

        data = sde.run_trajectory(dt,T,gen_force,noise_amp, x0 = phi0, v0 = v0)

        #new initial values
        phi0 = data[0,-1]
        v0 = data[1,-1]

        vmeans_backward[ib_steps-j-1] = np.sum(data[1])/N
        
        if j%50 == 0:
            print("backward "+str(j))
    
    print('')
    print("--- %s seconds ---" % (time.time() - start_time))

    return [vmeans_forward , vmeans_backward]

def sweep_forward_backward_KM(dt,T,v0,ib_max,ib_steps,F_of_ib,G):
    
    start_time = time.time() 

    #ib array and data container
    N = int(T/dt)
    ib_steps = int(ib_steps)
    ibs = np.linspace( -ib_max , ib_max , ib_steps) 
    vmeans_forward = np.zeros(ib_steps)
    vmeans_backward = np.zeros(ib_steps)

    #initial values (start with ohm velocity!)
    X0 = [0,v0,v0]
    
    print('')

    #forward sweep
    for j in range(ib_steps):

        F = lambda t,X: F_of_ib(ibs[j],t,X)
        
        Xs , DWs = sde.run_trajectory_general(dt,T,F,G,X0)

        #new initial values (s.t. hysteresis may be observed)
        X0 = Xs[-1]

        vmeans_forward[j] = np.sum(Xs.T[1])/N

        if j%50 == 0:
            print("forward "+str(j))

    #backward sweep
    for j in range(ib_steps):

        F = lambda t,X: F_of_ib(ibs[ib_steps-j-1],t,X)

        Xs , DWs = sde.run_trajectory_general(dt,T,F,G,X0)

        #new initial values
        X0 = Xs[-1]

        vmeans_backward[ib_steps-j-1] = np.sum(Xs.T[1])/N

        if j%50 == 0:
            print("backward "+str(j))

    print('')
    print("--- %s seconds ---" % (time.time() - start_time))

    return [vmeans_forward , vmeans_backward]